@@ -224,6 +224,19 @@ function test_bazel_version_from_base_url() {
       (echo "FAIL: Expected to find 'Build label: 7.2.1' in the output of 'bazelisk version'"; exit 1)
 }
 
+function test_bazel_version_rc_from_base_url() {
+  setup
+
+  echo "9.0.0rc1" > .bazelversion
+
+  BAZELISK_BASE_URL="https://github.com/bazelbuild/bazel/releases/download" \
+      BAZELISK_HOME="$BAZELISK_HOME" \
+          bazelisk version 2>&1 | tee log
+
+  grep "Build label: 9.0.0rc1" log || \
+      (echo "FAIL: Expected to find 'Build label: 9.0.0rc1' in the output of 'bazelisk version'"; exit 1)
+}
+
 function test_bazel_latest_minus_3_py() {
   setup
 
@@ -318,7 +331,7 @@ EOF
 function test_path_is_consistent_regardless_of_base_url() {
   setup
 
-  echo 8.3.0 > .bazelversion
+  echo 8.5.1 > .bazelversion
 
   cat >MODULE.bazel <<EOF
 print_path = use_repo_rule("//:print_path.bzl", "print_path")
@@ -534,6 +547,9 @@ if [[ $BAZELISK_VERSION == "GO" ]]; then
   test_bazel_version_from_base_url
   echo
 
+  echo "# test_bazel_version_rc_from_base_url"
+  test_bazel_version_rc_from_base_url
+
   echo "# test_bazel_version_from_user_home_bazeliskrc"
   test_bazel_version_from_user_home_bazeliskrc
   echo
