@@ -11,6 +11,7 @@ import (
 	"crypto/rand"
 	"crypto/sha256"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"log"
@@ -110,7 +111,10 @@ func RunBazeliskWithArgsFuncAndConfigAndOut(argsFunc ArgsFunc, repos *Repositori
 	// --print_env must be the first argument.
 	if len(args) > 0 && args[0] == "--print_env" {
 		// print environment variables for sub-processes
-		cmd := makeBazelCmd(bazelInstallation.Path, args, nil, config)
+		cmd, err := makeBazelCmd(bazelInstallation.Path, args, nil, config)
+		if err != nil {
+			return -1, err
+		}
 		for _, val := range cmd.Env {
 			fmt.Println(val)
 		}
@@ -167,12 +171,16 @@ func RunBazeliskWithArgsFuncAndConfigAndOut(argsFunc ArgsFunc, repos *Repositori
 	if isCompletionCommand(args) {
 		err := handleCompletionCommand(args, bazelInstallation, config)
 		if err != nil {
+			if errors.Is(err, httputil.NotFound) {
+				return -1, fmt.Errorf("The `completion` command is not supported on %s", platforms.PrettyLabel())
+			}
+
 			return -1, fmt.Errorf("could not handle completion command: %v", err)
 		}
 		return 0, nil
 	}
 
-	exitCode, err := runBazel(bazelInstallation.Path, args, out, config)
+	exitCode, err := runBazel(bazelInstallation.Path, args, out, config, isRunCommand(args))
 	if err != nil {
 		return -1, fmt.Errorf("could not run Bazel: %v", err)
 	}
@@ -618,20 +626,50 @@ func maybeDelegateToWrapperFromDir(bazel string, wd string, config config.Config
 	}
 
 	root := ws.FindWorkspaceRoot(wd)
-	wrapper := filepath.Join(root, wrapperPath)
-	if stat, err := os.Stat(wrapper); err == nil && !stat.Mode().IsDir() && stat.Mode().Perm()&0111 != 0 {
-		return wrapper
+
+	ext := platforms.DetermineExecutableFilenameSuffix()
+
+	// The list of candidate wrappers must go from most specific to least specific.
+	var candidates []string
+
+	osName, osErr := platforms.DetermineOperatingSystem()
+	// We pass platforms.DarwinArm64MinVersion to disable the Darwin x86_64 fallback because this feature
+	// was added years after Apple Silicon launched and it's not worth trying to be backwards compatible.
+	arch, archErr := platforms.DetermineArchitecture(osName, platforms.DarwinArm64MinVersion)
+
+	if osErr == nil && archErr == nil {
+		// OS- and architecture-specific wrapper candidates.
+		candidates = append(candidates, filepath.Join(root, wrapperPath+"."+osName+"-"+arch+ext))
 	}
+	if archErr == nil {
+		// architecture-specific wrapper candidates.
+		candidates = append(candidates, filepath.Join(root, wrapperPath+"."+arch+ext))
+	}
+	candidates = append(candidates, filepath.Join(root, wrapperPath+ext))
 
 	if runtime.GOOS == "windows" {
-		powershellWrapper := filepath.Join(root, wrapperPath+".ps1")
-		if stat, err := os.Stat(powershellWrapper); err == nil && !stat.Mode().IsDir() {
-			return powershellWrapper
+		candidates = append(candidates, filepath.Join(root, wrapperPath+".ps1"))
+		candidates = append(candidates, filepath.Join(root, wrapperPath+".bat"))
+		// It's left only for backward compatibility. Executable wrapper is supposed to have ".exe" extension on Windows
+		// and been probed earlier. Script wrappers are supposed to have proper extensions and should be started
+		// differently depending on them. Script wrapper without an extension is unreliable on Windows (in the same way
+		// as starting batch wrappers without an explicit cmd.exe usage, see #731 for details), so it should be avoided.
+		// But in some cases it can work and some users may rely on it, so let it be.
+		candidates = append(candidates, filepath.Join(root, wrapperPath))
+	}
+
+	for _, wrapper := range candidates {
+		stat, err := os.Stat(wrapper)
+		if err != nil {
+			continue
 		}
 
-		batchWrapper := filepath.Join(root, wrapperPath+".bat")
-		if stat, err := os.Stat(batchWrapper); err == nil && !stat.Mode().IsDir() {
-			return batchWrapper
+		valid := !stat.Mode().IsDir()
+		if runtime.GOOS != "windows" {
+			valid = valid && stat.Mode().Perm()&0111 != 0
+		}
+		if valid {
+			return wrapper
 		}
 	}
 
@@ -650,7 +688,7 @@ func maybeDelegateToWrapper(bazel string, config config.Config) string {
 func prependDirToPathList(cmd *exec.Cmd, dir string) {
 	found := false
 	for idx, val := range cmd.Env {
-		splits := strings.Split(val, "=")
+		splits := strings.SplitN(val, "=", 2)
 		if len(splits) != 2 {
 			continue
 		}
@@ -666,10 +704,22 @@ func prependDirToPathList(cmd *exec.Cmd, dir string) {
 	}
 }
 
-func makeBazelCmd(bazel string, args []string, out io.Writer, config config.Config) *exec.Cmd {
+func makeBazelCmd(bazel string, args []string, out io.Writer, config config.Config) (*exec.Cmd, error) {
 	execPath := maybeDelegateToWrapper(bazel, config)
 
-	cmd := exec.Command(execPath, args...)
+	var cmd *exec.Cmd
+	if execPath == bazel || runtime.GOOS != "windows" {
+		cmd = exec.Command(execPath, args...)
+	} else {
+		if strings.HasSuffix(execPath, ".ps1") {
+			cmd = makePowerShellScriptCmd(execPath, args)
+		} else if strings.HasSuffix(execPath, ".bat") {
+			cmd = makeBatchScriptCmd(execPath, args)
+		} else {
+			return nil, fmt.Errorf("unexpected wrapper type: %s", execPath)
+		}
+	}
+
 	cmd.Env = append(os.Environ(), skipWrapperEnv+"=true")
 	if execPath != bazel {
 		cmd.Env = append(cmd.Env, fmt.Sprintf("%s=%s", bazelReal, bazel))
@@ -686,11 +736,38 @@ func makeBazelCmd(bazel string, args []string, out io.Writer, config config.Conf
 		cmd.Stdout = out
 	}
 	cmd.Stderr = os.Stderr
-	return cmd
+	return cmd, nil
 }
 
-func runBazel(bazel string, args []string, out io.Writer, config config.Config) (int, error) {
-	cmd := makeBazelCmd(bazel, args, out, config)
+func isRunCommand(args []string) bool {
+	for _, arg := range args {
+		if arg == "--" {
+			return false
+		}
+		if arg == "run" {
+			return true
+		}
+	}
+	return false
+}
+
+func runBazel(bazel string, args []string, out io.Writer, config config.Config, useExecOnUnix bool) (int, error) {
+	cmd, makeCmdErr := makeBazelCmd(bazel, args, out, config)
+	if makeCmdErr != nil {
+		return 1, makeCmdErr
+	}
+	if useExecOnUnix && runtime.GOOS != "windows" {
+		execPath := cmd.Path
+		execArgs := cmd.Args
+		execEnv := cmd.Env
+		err := syscall.Exec(execPath, execArgs, execEnv)
+		if err != nil {
+			return 1, fmt.Errorf("could not exec Bazel: %v", err)
+		}
+		// This code is unreachable if exec succeeds
+		return 0, nil
+	}
+
 	err := cmd.Start()
 	if err != nil {
 		return 1, fmt.Errorf("could not start Bazel: %v", err)
@@ -709,7 +786,6 @@ func runBazel(bazel string, args []string, out io.Writer, config config.Config)
 	// by the terminal. As a side effect, we also suppress the printing of a
 	// Go stack trace upon receiving SIGQUIT, which is unhelpful as users tend
 	// to report it instead of the far more valuable Java thread dump.
-	// TODO(#512): We may want to treat a `bazel run` command differently.
 	// Since signal handlers are process-wide global state and bazelisk may be
 	// used as a library, reset the signal handlers after the process exits.
 	sigCh := make(chan os.Signal, 1)
@@ -720,7 +796,18 @@ func runBazel(bazel string, args []string, out io.Writer, config config.Config)
 	if err != nil {
 		if exitError, ok := err.(*exec.ExitError); ok {
 			waitStatus := exitError.Sys().(syscall.WaitStatus)
-			return waitStatus.ExitStatus(), nil
+			// it's only correct to use waitStatus.ExitStatus when the process terminated normally, i.e. waitStatus.Exited() == true
+			if waitStatus.Exited() {
+				return waitStatus.ExitStatus(), nil
+			}
+			// if the process was terminated by a signal on a POSIX-compatible system, let's report its exit code in the same way
+			// as shells do - as 128 + signal number.
+			// It's not a perfect solution, because information that a signal has terminated the process is lost,
+			// but at least we propagate a proper exit code.
+			if runtime.GOOS != "windows" && waitStatus.Signaled() {
+				return 128 + int(waitStatus.Signal()), nil
+			}
+			return 1, fmt.Errorf("unexpected wait status of Bazel: %v", waitStatus)
 		}
 		return 1, fmt.Errorf("could not launch Bazel: %v", err)
 	}
@@ -735,7 +822,7 @@ func getIncompatibleFlags(bazelPath, cmd string, config config.Config) ([]string
 	}
 
 	out := strings.Builder{}
-	if _, err := runBazel(bazelPath, []string{"help", cmd, "--short"}, &out, config); err != nil {
+	if _, err := runBazel(bazelPath, []string{"help", cmd, "--short"}, &out, config, false); err != nil {
 		return nil, fmt.Errorf("unable to determine incompatible flags with binary %s: %v", bazelPath, err)
 	}
 
@@ -811,7 +898,7 @@ func shutdownIfNeeded(bazelPath string, startupOptions []string, config config.C
 
 	args := append(startupOptions, "shutdown")
 	fmt.Printf("bazel %s\n", strings.Join(args, " "))
-	exitCode, err := runBazel(bazelPath, args, nil, config)
+	exitCode, err := runBazel(bazelPath, args, nil, config, false)
 	fmt.Printf("\n")
 	if err != nil {
 		log.Fatalf("failed to run bazel shutdown: %v", err)
@@ -830,7 +917,7 @@ func cleanIfNeeded(bazelPath string, startupOptions []string, config config.Conf
 
 	args := append(startupOptions, "clean", "--expunge")
 	fmt.Printf("bazel %s\n", strings.Join(args, " "))
-	exitCode, err := runBazel(bazelPath, args, nil, config)
+	exitCode, err := runBazel(bazelPath, args, nil, config, false)
 	fmt.Printf("\n")
 	if err != nil {
 		log.Fatalf("failed to run clean: %v", err)
@@ -1029,7 +1116,7 @@ func testWithBazelAtCommit(bazelCommit string, args []string, bazeliskHome strin
 	shutdownIfNeeded(bazelPath, startupOptions, config)
 	cleanIfNeeded(bazelPath, startupOptions, config)
 	fmt.Printf("bazel %s\n", strings.Join(args, " "))
-	bazelExitCode, err := runBazel(bazelPath, args, nil, config)
+	bazelExitCode, err := runBazel(bazelPath, args, nil, config, false)
 	if err != nil {
 		return -1, fmt.Errorf("could not run Bazel: %v", err)
 	}
@@ -1046,7 +1133,7 @@ func migrate(bazelPath string, baseArgs []string, flags []string, config config.
 	shutdownIfNeeded(bazelPath, startupOptions, config)
 	cleanIfNeeded(bazelPath, startupOptions, config)
 	fmt.Printf("bazel %s\n", strings.Join(args, " "))
-	exitCode, err := runBazel(bazelPath, args, nil, config)
+	exitCode, err := runBazel(bazelPath, args, nil, config, false)
 	if err != nil {
 		log.Fatalf("could not run Bazel: %v", err)
 	}
@@ -1061,7 +1148,7 @@ func migrate(bazelPath string, baseArgs []string, flags []string, config config.
 	shutdownIfNeeded(bazelPath, startupOptions, config)
 	cleanIfNeeded(bazelPath, startupOptions, config)
 	fmt.Printf("bazel %s\n", strings.Join(args, " "))
-	exitCode, err = runBazel(bazelPath, args, nil, config)
+	exitCode, err = runBazel(bazelPath, args, nil, config, false)
 	if err != nil {
 		log.Fatalf("could not run Bazel: %v", err)
 	}
@@ -1079,7 +1166,7 @@ func migrate(bazelPath string, baseArgs []string, flags []string, config config.
 		shutdownIfNeeded(bazelPath, startupOptions, config)
 		cleanIfNeeded(bazelPath, startupOptions, config)
 		fmt.Printf("bazel %s\n", strings.Join(args, " "))
-		exitCode, err = runBazel(bazelPath, args, nil, config)
+		exitCode, err = runBazel(bazelPath, args, nil, config, false)
 		if err != nil {
 			log.Fatalf("could not run Bazel: %v", err)
 		}
@@ -1158,7 +1245,7 @@ func handleCompletionCommand(args []string, bazelInstallation *BazelInstallation
 	// Get the completion script for the current Bazel version
 	completionScript, err := getBazelCompletionScript(bazelInstallation.Version, bazeliskHome, shell, config)
 	if err != nil {
-		return fmt.Errorf("could not get completion script: %v", err)
+		return fmt.Errorf("could not get completion script: %w", err)
 	}
 
 	fmt.Print(completionScript)
@@ -1188,7 +1275,7 @@ func getBazelCompletionScript(version string, bazeliskHome string, shell string,
 	// Download completion scripts if necessary (handles content-based caching internally)
 	installerHash, err := downloadCompletionScriptIfNecessary(installerURL, version, bazeliskHome, baseURL, config)
 	if err != nil {
-		return "", fmt.Errorf("could not download completion script: %v", err)
+		return "", fmt.Errorf("could not download completion script: %w", err)
 	}
 
 	// Read the requested completion script using installer content hash
