@@ -7,6 +7,7 @@ import (
 	"fmt"
 	"log"
 	"os"
+	"os/exec"
 	"path/filepath"
 	"runtime"
 	"strings"
@@ -64,29 +65,96 @@ func TestMaybeDelegateToNoNonExecutableWrapper(t *testing.T) {
 	}
 }
 
-func TestMaybeDelegateToStandardWrapper(t *testing.T) {
-	// It's not guaranteed that `tools/bazel` is executable on the
-	// Windows host running this test. Thus the test is skipped on
-	// this platform to guarantee consistent results.
-	if runtime.GOOS == "windows" {
-		return
-	}
-
-	var tmpDir, err = os.MkdirTemp("", "TestMaybeDelegateToStandardWrapper")
+func TestMaybeDelegateToOsAndArchSpecificWrapper(t *testing.T) {
+	var tmpDir, err = os.MkdirTemp("", "TestMaybeDelegateToOsAndArchSpecificWrapper")
 	if err != nil {
 		log.Fatal(err)
 	}
 	defer os.RemoveAll(tmpDir)
 
+	osName, err := platforms.DetermineOperatingSystem()
+	if err != nil {
+		log.Fatal(err)
+	}
+	arch, err := platforms.DetermineArchitecture(osName, platforms.DarwinArm64MinVersion)
+	if err != nil {
+		log.Fatal(err)
+	}
+
+	ext := platforms.DetermineExecutableFilenameSuffix()
+
 	os.MkdirAll(tmpDir, os.ModeDir|0700)
 	os.WriteFile(filepath.Join(tmpDir, "WORKSPACE"), []byte(""), 0600)
 	os.WriteFile(filepath.Join(tmpDir, "BUILD"), []byte(""), 0600)
 
 	os.MkdirAll(filepath.Join(tmpDir, "tools"), os.ModeDir|0700)
-	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"), []byte(""), 0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel."+osName+"-"+arch+ext), []byte(""), 0700)
+	// Also create the standard wrapper to ensure we prefer the os/arch-specific wrapper.
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"+ext), []byte(""), 0700)
+	// Also create the plaform-specific wrapper to ensure we prefer the os/arch-specific wrapper.
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel."+arch+ext), []byte(""), 0700)
 
 	entrypoint := maybeDelegateToWrapperFromDir("bazel_real", tmpDir, config.Null())
-	expected := filepath.Join(tmpDir, "tools", "bazel")
+	expected := filepath.Join(tmpDir, "tools", "bazel."+osName+"-"+arch+ext)
+
+	if entrypoint != expected {
+		t.Fatalf("Expected to delegate bazel to %q, but got %q", expected, entrypoint)
+	}
+}
+
+func TestMaybeDelegateToArchSpecificWrapper(t *testing.T) {
+	var tmpDir, err = os.MkdirTemp("", "TestMaybeDelegateToArchSpecificWrapper")
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer os.RemoveAll(tmpDir)
+
+	osName, err := platforms.DetermineOperatingSystem()
+	if err != nil {
+		log.Fatal(err)
+	}
+	arch, err := platforms.DetermineArchitecture(osName, platforms.DarwinArm64MinVersion)
+	if err != nil {
+		log.Fatal(err)
+	}
+
+	ext := platforms.DetermineExecutableFilenameSuffix()
+
+	os.MkdirAll(tmpDir, os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "WORKSPACE"), []byte(""), 0600)
+	os.WriteFile(filepath.Join(tmpDir, "BUILD"), []byte(""), 0600)
+
+	os.MkdirAll(filepath.Join(tmpDir, "tools"), os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel."+arch+ext), []byte(""), 0700)
+	// Also create the standard wrapper to ensure we prefer the arch-specific wrapper.
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"+ext), []byte(""), 0700)
+
+	entrypoint := maybeDelegateToWrapperFromDir("bazel_real", tmpDir, config.Null())
+	expected := filepath.Join(tmpDir, "tools", "bazel."+arch+ext)
+
+	if entrypoint != expected {
+		t.Fatalf("Expected to delegate bazel to %q, but got %q", expected, entrypoint)
+	}
+}
+
+func TestMaybeDelegateToStandardWrapper(t *testing.T) {
+	var tmpDir, err = os.MkdirTemp("", "TestMaybeDelegateToStandardWrapper")
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer os.RemoveAll(tmpDir)
+
+	ext := platforms.DetermineExecutableFilenameSuffix()
+
+	os.MkdirAll(tmpDir, os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "WORKSPACE"), []byte(""), 0600)
+	os.WriteFile(filepath.Join(tmpDir, "BUILD"), []byte(""), 0600)
+
+	os.MkdirAll(filepath.Join(tmpDir, "tools"), os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"+ext), []byte(""), 0700)
+
+	entrypoint := maybeDelegateToWrapperFromDir("bazel_real", tmpDir, config.Null())
+	expected := filepath.Join(tmpDir, "tools", "bazel"+ext)
 
 	if entrypoint != expected {
 		t.Fatalf("Expected to delegate bazel to %q, but got %q", expected, entrypoint)
@@ -175,6 +243,66 @@ func TestMaybeDelegateToPowershellOverBatchWrapper(t *testing.T) {
 	}
 }
 
+func TestMaybeDelegateToPowershellOverNoExtensionWrapper(t *testing.T) {
+	tmpDir, err := os.MkdirTemp("", "TestMaybeDelegateToPowershellOverBatchWrapper")
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer os.RemoveAll(tmpDir)
+
+	os.MkdirAll(tmpDir, os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "WORKSPACE"), []byte(""), 0600)
+	os.WriteFile(filepath.Join(tmpDir, "BUILD"), []byte(""), 0600)
+
+	os.MkdirAll(filepath.Join(tmpDir, "tools"), os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel.ps1"), []byte(""), 0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"), []byte(""), 0700)
+
+	entrypoint := maybeDelegateToWrapperFromDir("bazel_real", tmpDir, config.Null())
+
+	// On Windows a PowerShell script should be chosen over the wrapper without an extension
+	var expected string
+	if runtime.GOOS == "windows" {
+		expected = filepath.Join(tmpDir, "tools", "bazel.ps1")
+	} else {
+		expected = filepath.Join(tmpDir, "tools", "bazel")
+	}
+
+	if entrypoint != expected {
+		t.Fatalf("Expected to delegate bazel to %q, but got %q", expected, entrypoint)
+	}
+}
+
+func TestMaybeDelegateToBatchOverNoExtensionWrapper(t *testing.T) {
+	tmpDir, err := os.MkdirTemp("", "TestMaybeDelegateToPowershellOverBatchWrapper")
+	if err != nil {
+		log.Fatal(err)
+	}
+	defer os.RemoveAll(tmpDir)
+
+	os.MkdirAll(tmpDir, os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "WORKSPACE"), []byte(""), 0600)
+	os.WriteFile(filepath.Join(tmpDir, "BUILD"), []byte(""), 0600)
+
+	os.MkdirAll(filepath.Join(tmpDir, "tools"), os.ModeDir|0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel.bat"), []byte(""), 0700)
+	os.WriteFile(filepath.Join(tmpDir, "tools", "bazel"), []byte(""), 0700)
+
+	entrypoint := maybeDelegateToWrapperFromDir("bazel_real", tmpDir, config.Null())
+
+	// On Windows a batch script should be chosen over the wrapper without an extension
+	var expected string
+	if runtime.GOOS == "windows" {
+		expected = filepath.Join(tmpDir, "tools", "bazel.bat")
+	} else {
+		expected = filepath.Join(tmpDir, "tools", "bazel")
+	}
+
+	if entrypoint != expected {
+		t.Fatalf("Expected to delegate bazel to %q, but got %q", expected, entrypoint)
+	}
+}
+
 // Completion Tests
 
 func TestIsCompletionCommand(t *testing.T) {
@@ -661,3 +789,35 @@ exit 0
 		t.Errorf("Fish completion content mismatch")
 	}
 }
+
+func TestPrependDirToPathListWithEqualsInPath(t *testing.T) {
+	cmd := &exec.Cmd{
+		Env: []string{
+			"PATH=/path/with=equals:/another/path",
+			"OTHER_VAR=value",
+		},
+	}
+
+	prependDirToPathList(cmd, "/new/dir")
+
+	// Check that PATH was properly updated
+	found := false
+	for _, env := range cmd.Env {
+		if strings.HasPrefix(env, "PATH=") {
+			found = true
+			// Should contain the new directory at the beginning
+			if !strings.Contains(env, "/new/dir") {
+				t.Errorf("Expected PATH to contain '/new/dir', got: %s", env)
+			}
+			// Should contain the original PATH value
+			if !strings.Contains(env, "/path/with=equals:/another/path") {
+				t.Errorf("Expected PATH to contain original value, got: %s", env)
+			}
+			break
+		}
+	}
+
+	if !found {
+		t.Error("PATH environment variable not found")
+	}
+}
